/// <reference types="react" />
import * as React from 'react';
import { IProcessedStyleSet, IStyleSet, ITheme } from '@uifabric/styling';
import { IStyleFunctionOrObject } from '@uifabric/utilities';
export declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
/**
 * Optional props for styleable components. If these props are present, they will automatically be
 * used by Foundation when applying theming and styling.
 */
export interface IStyleableComponentProps<TViewProps, TStyleSet extends IStyleSet<TStyleSet>> {
    styles?: IStyleFunctionOrObject<TViewProps, TStyleSet>;
    theme?: ITheme;
}
/**
 * Props added by Foundation for styles functions.
 */
export interface IStyledProps<TTheme> {
    theme: TTheme;
}
/**
 * Enforce props contract on state components, including the view prop and its shape.
 */
export declare type IStateComponentProps<TComponentProps, TViewProps> = TComponentProps & {
    renderView: React.StatelessComponent<TViewProps>;
};
/**
 * Imposed state component props contract with styling props as well as a renderView
 * prop that the StateComponent should make use of in its render output (and should be its only render output.)
 */
export declare type IStateComponentType<TComponentProps, TViewProps> = React.ComponentType<IStateComponentProps<TComponentProps, TViewProps>>;
/**
 * The props that get passed to view components.
 */
export declare type IViewComponentProps<TViewProps, TProcessedStyleSet> = TViewProps & {
    classNames: TProcessedStyleSet;
};
/**
 * A helper type for defining view components, including its properties.
 */
export declare type IViewComponent<TViewProps, TProcessedStyleSet> = React.StatelessComponent<IViewComponentProps<TViewProps, TProcessedStyleSet>>;
/**
 * Component used by foundation to tie elements together.
 * @see createComponent for generic type documentation.
 */
export interface IComponentOptions<TComponentProps, TViewProps, TStyleSet extends IStyleSet<TStyleSet>, TStatics = {}> {
    /**
     * Display name to identify component in React hierarchy.
     */
    displayName: string;
    /**
     * List of fields which can be customized.
     */
    fields?: string[];
    /**
     * Styles prop to pass into component.
     */
    styles?: IStyleFunctionOrObject<TViewProps & IStyledProps<ITheme>, TStyleSet>;
    /**
     * React view stateless component.
     */
    view: IViewComponent<TViewProps, IProcessedStyleSet<TStyleSet>>;
    /**
     * Optional state component that processes TComponentProps into TViewProps.
     */
    state?: IStateComponentType<TComponentProps, TViewProps>;
    /**
     * Optional static object to assign to constructed component.
     */
    statics?: TStatics;
}
/**
 * Variant of IComponentOptions for stateful components with appropriate typing and required properties.
 */
export declare type IComponent<TComponentProps, TViewProps, TStyleSet extends IStyleSet<TStyleSet>, TStatics = {}> = IComponentOptions<TComponentProps, TViewProps, TStyleSet, TStatics> & Required<Pick<IComponentOptions<TComponentProps, TComponentProps, TStyleSet, TStatics>, 'state'>>;
/**
 * Variant of IComponentOptions for stateless components with appropriate typing and required properties.
 */
export declare type IStatelessComponent<TComponentProps, TStyleSet extends IStyleSet<TStyleSet>, TStatics = {}> = Omit<IComponentOptions<TComponentProps, TComponentProps, TStyleSet, TStatics>, 'state'>;
/**
 * Assembles a higher order component based on the following: styles, theme, view, and state.
 * Imposes a separation of concern and centralizes styling processing to increase ease of use and robustness
 * in how components use and apply styling and theming.
 *
 * Automatically merges and applies themes and styles with theme / styleprops having the highest priority.
 * State component, if provided, is passed in props for processing. Props from state / user are automatically processed
 * and styled before finally being passed to view.
 *
 * State components should contain all stateful behavior and should not generate any JSX, but rather simply call the view prop.
 * Views should simply be stateless pure functions that receive all props needed for rendering their output.
 * State component is optional. If state not provided, created component is essentially a functional stateless component.
 *
 * TComponentProps: A styleable props interface for the created component.
 * TViewProps: The props specific to the view, including processed properties outputted by optional state component. If state
 * component is not provided, TComponentProps is the same as TViewProps.
 * TStyleSet: The type for styles properties.
 * TProcessedStyleSet: The type provided by mergeStyleSets provider after processing TStyleSet and provided to views.
 * TTheme: The type for theme properties as well as the getTheme provider.
 * TScheme: The type for identifying schemes.
 * TStatics: Static type for statics applied to created component object.
 *
 * @param {IComponent} component
 * @param {IComponentProviders} providers
 */
export declare function createComponent<TComponentProps, TViewProps, TStyleSet extends IStyleSet<TStyleSet>, TStatics = {}>(component: IComponent<TComponentProps, TViewProps, TStyleSet, TStatics>): React.StatelessComponent<TComponentProps> & TStatics;
/**
 * A wrapper function around createComponent to confine generics and component properties for stateless components.
 *
 * @see {@link createComponent} for more information.
 */
export declare function createStatelessComponent<TComponentProps, TStyleSet extends IStyleSet<TStyleSet>, TStatics = {}>(component: IStatelessComponent<TComponentProps, TStyleSet, TStatics>): React.StatelessComponent<TComponentProps> & TStatics;
